#!/bin/sh -eu

#
# This script runs autopilot
#

# Copyright © 2013 Canonical Ltd.
# Author: Jean-baptiste Lallement <jean-baptiste.lallement@canonical.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

# Required to display stdout in ~/.xsession-error

TESTBASE=/var/local/autopilot/
AP_ARTIFACTS=$TESTBASE/videos/
AP_RESULTS=$TESTBASE/junit/
AP_TESTSUITES=$TESTBASE/testsuites
AP_LOGFILE=$TESTBASE/autopilot.log
SPOOLDIR=$TESTBASE/spool
AP_OPTS="-v -r -rd $AP_ARTIFACTS -f xml"
# TESTING ONLY -- REcording is disabled
#AP_OPTS="-v -f xml"
OTTO_SUMMARY=/var/local/otto/summary.log
MEMUSAGE_CMD=/usr/local/bin/capture_procstat

# Define general configuration files 
[ -f $TESTBASE/config ] && . $TESTBASE/config

setup_tests() {
    # Prepares the environment for the tests
    flag=$HOME/.ap_setup_done

    [ -e "$flag" ] && return 0

    # Disable notifications and screensaver
    gsettings set com.ubuntu.update-notifier show-apport-crashes false
    gsettings set org.gnome.desktop.screensaver idle-activation-enabled false

    # Loads the list of test and queue them in test spool
    sudo mkdir -p $SPOOLDIR $AP_ARTIFACTS $AP_RESULTS $(dirname $OTTO_SUMMARY)
    sudo chown -R $USER:$USER $TESTBASE $SPOOLDIR $AP_ARTIFACTS $AP_RESULTS $(dirname $OTTO_SUMMARY)

    if [ -e "$AP_TESTSUITES" ]; then
        (cd $SPOOLDIR; touch $(cat $AP_TESTSUITES))
    fi
    touch $flag
}

run_tests() {
    # Runs all the tests in spooldir
    #
    # $1: Spool directory
    spooldir=$1
    if [ ! -d $spooldir ]; then
        echo "E: '$spooldir is not a directory. Exiting!"
        exit 1
    fi

    if ! which autopilot >/dev/null 2>&1; then
        echo "E: autopilot is required to run autopilot tests"
        echo "autopilot_installed (see autopilot.log for details): ERROR" >> $OTTO_SUMMARY
        sudo shutdown -h now
        exit 1
    fi
    echo "autopilot_installed: PASS" >> $OTTO_SUMMARY

    exec >>$AP_LOGFILE
    exec 2>&1

    # Capture memusage
    $MEMUSAGE_CMD -o $TESTBASE/memusage.log 1 &
    [ $? -gt 0 ] && echo "W: Failed to start $MEMUSAGE_CMD ."

    for testfile in $(ls -d $spooldir/* 2>/dev/null); do
        testname=$(basename $testfile)
        # We don't want to fail if AP fail but we want the return code
        set +e  
        echo "I: Running autopilot run $testname $AP_OPTS -o $AP_RESULTS/$testname.xml"
        autopilot run $testname $AP_OPTS -o $AP_RESULTS/$testname.xml 
        AP_RC=$?
        if [ $AP_RC -gt 0 ]; then
            echo "${testname}: FAIL" >> $OTTO_SUMMARY
        else
            echo "${testname}: DONE" >> $OTTO_SUMMARY
        fi
        set -e
        sudo rm -f $testfile
    done

    if [ ! "$(ls -A $spooldir/)" ]; then
        echo "I: No test left to run"
        sudo shutdown -h now
    fi
}

setup_tests
run_tests $SPOOLDIR
